// Fragment shader for per-pixel Phong interpolation and shading.

// The "varying" keyword means that the parameter's value is interpolated
// between the nearby vertices.
varying vec3 N;
varying vec3 v;

uniform float brightness;

const int NUM_LIGHTS = 2; // directional and point

void main()
{
    // The scene's ambient light.
    vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;

	// The normal vectors is generally not normalized after being
	// interpolated across a triangle.  Here we normalize it.
	vec3 Normal = normalize(N);

	// Since the vertex is in eye space, the direction to the
	// viewer is simply the normalized vector from v to the
	// origin.
	vec3 Viewer = -normalize(v);

    gl_FragColor = globalAmbient + gl_FrontMaterial.emission;

	for (int i = 0; i < NUM_LIGHTS; i++) {
        vec3 Light = normalize(gl_LightSource[i].position.xyz - (i == 0 ? vec3(0,0,0) : v));
	    vec3 Half = normalize(Viewer + Light);
	    float B = dot(Normal, Light) < 0.0 ? 0.0 : 1.0;

        float diffuseShade =  max(dot(Normal, Light), 0.0);
        float specularShade = B * pow(max(dot(Half, Normal), 0.0), gl_FrontMaterial.shininess);

	    float a = gl_LightSource[i].constantAttenuation;
	    float b = gl_LightSource[i].linearAttenuation;
	    float c = gl_LightSource[i].quadraticAttenuation;
	    float r = length(vec3(gl_LightSource[i].position - v)); 
	    float attenuatingCoefficient = i == 0 ? 1.0 : 
                                                (1.0 / (a + b*r + c*r*r));

        vec4 diffuse = diffuseShade * gl_FrontLightProduct[i].diffuse;
        vec4 specular = specularShade * gl_FrontLightProduct[i].specular;
        vec4 intensity = (diffuse + specular + gl_FrontLightProduct[i].ambient) * attenuatingCoefficient;
        
        gl_FragColor += intensity;
    }

	gl_FragColor = gl_FragColor * brightness;
}